package model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import util.Card;
import util.CardGroup;

/**
 * Models a hand cards. The hand is kept sorted in the natural
 * sorting order of the card. Does not have to be 
 * thread safe. The hand supports duplicates: adding the same card twice will result
 * in two additions.
 */
public class Hand implements Iterable<Card>
{
	/**
	 * Creates a new, empty hand.
	 */
	public Hand()
	{}
	
	/**
	 * Adds card to the Hand.
	 * @param card The card to add.
	 * @pre card != null
	 */
	public void add(Card card)
	{}
	
	/**
	 * Remove one instance of card from the hand. Does nothing if
	 * card is not in the hand.
	 * @param card The card to remove.
	 * @pre pCard != null
	 */
	public void remove(Card card)
	{}
	
	/**
	 * Removes all the cards from the hand.
	 * @post size() == 0
	 */
	public void clear()
	{}
	
	/**
	 * @return The number of cards in the hand.
	 */
	public int size()
	{}
	
	/**
	 * Determines if card is already in the hand.
	 * @param card The card to check.
	 * @return true if the card is already in the hand.
	 * @pre pCard != null
	 */
	public boolean contains(Card card)
	{}

	/**
	 * Returns false if group is a Joker. Otherwise,
	 * Returns true if this hand can play a legal move
	 * following the group being played. For a legal move
	 * to be possible, this hand must contain either a) a Joker; b) 
	 * a group of cards of same rank as group and of same or higher
	 * cardinality; c) if the rank of group is less than 2, a group
	 * of twos of at least group.size() -1. This method has no
	 * side effects.
	 * @param group the group to check against
	 * @return True if this can can play following group
	 */
	public boolean canPlayOn(CardGroup group)
	{}
	
	/**
	 * Removes cards from the hand and construct a card group.
	 * @param cards The cards to play
	 * @return The CardGroup to play
	 * @throws IllegalArgumentException if the cards are not in the hand or
	 * do not form a proper group.
	 * @pre cards !=null
	 * @post cards in cardgroup are removed from hand
	 */
	public CardGroup playGroup(List<Card> cards) throws IllegalArgumentException
	{}
	
	/**
	 * @return A CardGroup for every legal move from this hand after group
	 * has been played, or an empty array if no move is possible. This
	 * method has no side effect.
	 * @param group An array of CardGroup representing legal moves.
	 */
	public CardGroup[] playChoices(CardGroup group)
	{}
}
